/*
 * MSP430_SWI2C_Master.c
 */

//******************************************************************************
//  MSP430F20xx - I2C Master Transmitter and Receiver via CPU Software (GPIO)
//
//  Description: This code library configures the MSP430 as an I2C master device
//  capable of transmitting and receiving bytes using GPIO pins.  Specific I/O
//  pins can be selected in the corresponding header file.  By default, the same
//  pins that are used by the USI module are selected.
//
//                  Master
//                  MSP430
//             -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//            |                 |
//            |                 |
//            |         SDA/P2.7|-------> [I2C SLAVE SDA]
//            |         SCL/P2.6|-------> [I2C SLAVE SCL]
//
//  Note: Internal pull-ups are NOT used for SDA & SCL [DISABLED]
//
//  R. Wu
//  Texas Instruments Inc.
//  March 2008
//
// angepasst an MSP430G2553, uALFAT-SD, BMP085, ADXL345B
//	16.08.2012-06.10.2012 DL9MWE
//******************************************************************************

#include "MSP430_SWI2C_Master.h"

void MSP430_SWI2CMST_init(void)
{
  PxSEL &= ~(SCL | SDA);                    // Set GPIO function
  PxDIR |= (SCL | SDA);                     // Set output direction
  PxOUT &= ~(SCL | SDA);                    // SCL & SDA = 0 => low when = outputs
  PxDIR &= ~(SCL | SDA);                    // set to input, SCL & SDA = 1
}

// Set up start condition for I2C
void MSP430_SWI2CMST_start(void)
{
	PxDIR |= SDA;		// Set to output, data low [SDA = 0]
	_NOP();
	_NOP();				// Quick delay
	PxDIR |= SCL;		// Set to output, clock low [SCL = 0]
}

// Master sendet ein Byte an den I2C-Slave
void MSP430_SWI2CMST_txByte(unsigned char data)
{
  unsigned char bits, temp;

	temp = data;
	bits = 0x08;				// Load I2C bit counter
	while (bits != 0x00) {		// Loop until all bits are shifted
		if (temp & BIT7)		// Test data bit
			PxDIR &= ~SDA;		// Set to input, SDA = 1 via pull-up
		else
			PxDIR |= SDA;		// Set to output, data low [SDA = 0]
		PxDIR &= ~SCL;			// Set to input, clock high [SCL = 1]
		temp = (temp << 1);		// Shift bits 1 place to the left
		_NOP();					// Quick delay
		PxDIR |= SCL;			// Set to output, clock low [SCL = 0]
		bits = (bits - 1);		// Loop until 8 bits are sent
	}
	PxDIR &= ~SDA;				// Set to input, Slave setzt SDA
}


// set I2C_clock_pulse
void MSP430_SWI2C_clockpulse(void) {
	_NOP();
	_NOP();
	PxDIR &= ~SCL;			// Set to input, SCL = 1 via pull-up
	_NOP();
	_NOP();					// delay to meet I2C spec
	PxDIR |= SCL;			// Set to output, clock low [SCL = 0]
}

// Send Master Acknowledge
void MSP430_SWI2CMST_ack(void)
{
	PxDIR |= SDA;			// Set to output, data low [SDA = 0]
	MSP430_SWI2C_clockpulse();
	PxDIR &= ~SDA;			// Set SDA to input
}

// Send negative acknowledge
void MSP430_SWI2CMST_nack(void)
{
	PxDIR &= ~SDA;			// Set to input, SDA = 1 via pull-up
	MSP430_SWI2C_clockpulse();
}

// Get Slave Acknowledge
unsigned int MSP430_SWI2CSLV_ack(void)
{
  unsigned int error;
   PxDIR &= ~SCL;			// Set to input, SCL = 1 via pull-up
   _NOP();					// delay to meet I2C spec
   _NOP();					//   "        "        "
   if (PxIN & SDA)
	   error = 1;
   else
	   error = 0;
   PxDIR |= SCL;			// Set to output, clock low [SCL = 0]
   return error;
}


// liest ein Byte via I2C ein
unsigned char MSP430_SWI2CMST_rxByte(void)  // Read 8 bits of I2C data
{
  unsigned char bits, temp = 0;            // I2C bit counter
   bits = 0x08;                             // Load I2C bit counter
   while (bits > 0)                         // Loop until all bits are read
   {
      PxDIR &= ~SCL;                        // Set to input, SCL = 1 via pull-up
      temp = (temp << 1);                   // Shift bits 1 place to the left
      if (PxIN & SDA)                       // Check digital input
        temp = (temp + 1);                  // If input is 'H' store a '1'
      PxDIR |= SCL;                         // Set to output, data low [SCL = 0]
      bits = (bits - 1);                    // Decrement I2C bit counter
   }
   return (temp);                           // Return 8-bit data byte
}

// Send I2C stop command
void MSP430_SWI2CMST_stop(void) {
  PxDIR |= SDA;                             // Set to output, data low [SCA = 0]
  _NOP();                                   // Quick delay
  _NOP();                                   // Quick delay
  PxDIR &= ~SCL;                            // Set to input, SCL = 1 via pull-up
  _NOP();                                   // Quick delay
  _NOP();                                   // Quick delay
  PxDIR &= ~SDA;                            // Set to input, SDA = 1 via pull-up
}


// adressiert das I2C-Device und setzt eine Register-adresse
void I2C_set_register(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned int* error_code) {
	MSP430_SWI2CMST_start();						// Send I2C Start Condition
	MSP430_SWI2CMST_txByte(I2C_device_address);		// [ADDR] + R/W bit = 0
	*error_code = MSP430_SWI2CSLV_ack();			// get slave acknowledge (ACKS)
	MSP430_SWI2CMST_txByte(I2C_register_address);	// send ADXL345B Register Address
	*error_code += MSP430_SWI2CSLV_ack();			// get ACKS
}


// unsigned char aus I2C-Device-Register lesen
unsigned char I2C_read_char(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned int* error_code) {
  unsigned char data;
	I2C_set_register(I2C_device_address, I2C_register_address, error_code);
	MSP430_SWI2CMST_stop();
	MSP430_SWI2CMST_start();						// repeated start condition
	MSP430_SWI2CMST_txByte(I2C_device_address + 1);	// [ADDR] + R/W bit = 1
	*error_code = MSP430_SWI2CSLV_ack();			// get ACKS
	data = MSP430_SWI2CMST_rxByte();
	return data;
}


// *****************************************************************************
// ************************************ API ************************************

// schreibt einen 8-Bit-Wert in ein Register des I2C-Devices
void I2C_write_register_value(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned char register_value, unsigned int* error_code) {
	I2C_set_register(I2C_device_address, I2C_register_address, error_code);
	MSP430_SWI2CMST_txByte(register_value); 	// write register value
	*error_code += MSP430_SWI2CSLV_ack();		// get ACKS
	MSP430_SWI2CMST_stop();
}

// liest 8-Bits aus einem Register des I2C-Devices aus (single byte read)
unsigned char I2C_read_8Bit(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned int* error_code) {
  unsigned char data;
	data = I2C_read_char(I2C_device_address, I2C_register_address, error_code);
	MSP430_SWI2CMST_nack();
	MSP430_SWI2CMST_stop();
	return data;
}

// liest 16-Bits aus zwei aufeinanderfolgenden Registern des I2C-Devices aus (multiple byte read)
//  falls msbfirst = 1, ist das erste Byte das hherwertige, das zweite das niederwertige, sonst umgekehrt
unsigned int I2C_read_16Bit(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned char msbfirst, unsigned int* error_code) {
  unsigned char firstbyte, secondbyte;
	firstbyte = I2C_read_char(I2C_device_address, I2C_register_address, error_code);
	MSP430_SWI2CMST_ack();
	secondbyte = MSP430_SWI2CMST_rxByte();
	MSP430_SWI2CMST_nack();
	MSP430_SWI2CMST_stop();
	if(msbfirst)
		return ((unsigned int)firstbyte << 8) + (unsigned int)secondbyte;
	return ((unsigned int)secondbyte << 8) + (unsigned int)firstbyte;
}

// Ende MSP430_SWI2C_Master.c
